<?php 

/**
 * @package ARTIO Booking
 * @copyright Copyright (C) ARTIO s.r.o. All rights reserved.
 * @author ARTIO s.r.o., http://www.artio.net
 */

defined('_JEXEC') or die('Restricted access');

class com_bookingInstallerScript
{
	
	/**
	 * Install extensions. Set up DB, ACL and config. Included in unpacked package in temporary folder. 
	 */
	public function install()
	{
		JFactory::getLanguage()->load('com_booking', JPATH_ADMINISTRATOR);
		
		JLoader::import('admin.defines', dirname(__FILE__));
		JLoader::import('admin.helpers.installer', dirname(__FILE__));
		
		AInstaller::install();
		AInstallerJoomFish::install();
		
		JFile::copy(JPath::clean(dirname(__FILE__) . '/admin/extensions/payment_methods/paymentmethod.php'), JPath::clean(JPATH_ROOT . '/plugins/bookingpayment/paymentmethod.php'));
		
		$this->_db();
		$this->_cfg();
	}
	
	/**
	 * Uninstall extensions. Included in component administrator folder.
	 */
	public function uninstall()
	{
		JFactory::getLanguage()->load('com_booking', JPATH_ADMINISTRATOR);
		
		JLoader::import('helpers.installer', dirname(__FILE__));
		
		AInstaller::uninstall();
		AInstallerJoomFish::uninstall();
		
		JFile::delete(JPath::clean(JPATH_ROOT . '/plugins/bookingpayment/paymentmethod.php'));
	}
	
	/**
	 * Show welcome page. Included in unpacked package in temporary folder.
	 */
	function postflight()
	{
		$this->_acl();
		
		echo '<div class="install" style="background-color: #FFF;padding: 20px;">';
		echo '<a href="index.php?option=com_booking" title="">';
		echo '<img src="'.JURI::root().'components/com_booking/assets/images/logo-150.png" alt="" style="float: left;padding-right: 20px;" />';
		echo '</a>';
		echo '<h2 style="color: #0B55C4;font-size: 30px;line-height: 30px;margin: 0;padding: 0 0 10px 170px;">';
		echo '<a href="index.php?option=com_booking" title="">ARTIO Booking</a>';
		echo '</h2>';
		echo '<p style="font-size: 14px;margin: 0;padding: 0 0 10px 170px;width: 50%;">' . JText::_('BOOKING_DESCRIPTION') . '</p>';
		echo '<p style="font-size: 14px;margin: 0;padding: 0 0 10px 170px;width: 50%;">' . str_replace('"_QQ_"', '"', JText::_('SAMPLE_DATA')) . '</p>';
		echo '</div>';
	}

	/**
	 * Allow basic rule list for each user.
	 */
	private function _acl()
	{
		$db = JFactory::getDbo();
		 
		$rule = array();
		$rule[] = 'booking.reservation.create';
		$rule[] = 'booking.edit.customer';
		$rule[] = 'booking.mailing.new.reservation';
		$rule[] = 'booking.mailing.change.reservation';
		$rule[] = 'booking.mailing.cancel.reservation';
		 
		// load existing rules
		$query = $db->getQuery(true);
		$query->select('rules')->from('#__assets')->where("name = 'com_booking'");
		$db->setQuery($query);
		$rules = $db->loadResult();
		 
		$rules = json_decode($rules);
		 
		// initialize with fresh installation
		if (!is_object($rules))
			$rules = new stdClass();
		 
		foreach ($rule as $r)
			if (!isset($rules->$r))
			$rules->$r = new stdClass();
		 
		// load all usergroups
		$query = $db->getQuery(true);
		$query->select('id')->from('#__usergroups');
		$db->setQuery($query);
		$userGroups = $db->loadColumn();
		 
		// allow rule list for each usergroup
		foreach ($userGroups as $userGroup)
			foreach ($rule as $r)
				$rules->$r->$userGroup = 1;
		 
		// save rules back
		$rules = json_encode($rules);
		$rules = $db->quote($rules);
		 
		$query = $db->getQuery(true);
		$query->update('#__assets')->set("rules = $rules")->where("name = 'com_booking'");
		$db->setQuery($query);
		$db->query();
	}
	
	/**
	 * Setup default configuration with fresh installation.
	 */
	private function _cfg()
	{
		$db = JFactory::getDbo();
	
		if ($db->setQuery('SELECT COUNT(*) FROM #__booking_config')->loadResult() === '0') { // fresh installation, config is completely empty
			
			$config = JFactory::getXML(JPath::clean(dirname(__FILE__) . '/admin/config.xml'));
			$items = array();
			
			foreach ($config->fields->fieldset as $fieldset) // search for default values
				foreach ($fieldset as $field)
					if (isset($field['default']) && JString::trim((string) $field['name']) && JString::trim((string) $field['default']))
						$items[(string) $field['name']] = (string) $field['default'];
			
			$items['mailing_manager'] = JFactory::getApplication()->getCfg('mailfrom');
			
			$query = $db->getQuery(true);
			$query->insert('`#__booking_config`');
			$query->columns('`key`, `value`');
			
			foreach ($items as $name => $default)
				$query->values($db->quote($name) . ', ' . $db->quote($default));
			
			$db->setQuery($query)->query();
			
			JModelLegacy::addIncludePath(JPath::clean(dirname(__FILE__) . '/admin/models'));
			$model = JModelLegacy::getInstance('Config', 'BookingModel');
			/* @var $model BookingModelConfig */
			$model->setAdmins();
		}
	}
	
	public function db()
	{
		com_bookingInstallerScript::_db(JPath::clean(dirname(__FILE__).'/sql/install.mysql.utf8.sql'));
	}
	
	/**
	 * Update existing database table list during reinstall.
	 */
	private function _db($sql = null)
	{
		$db = JFactory::getDBO();
		$queries = JFile::read($sql ? $sql : JPath::clean(dirname(__FILE__) . '/admin/sql/install.mysql.utf8.sql')); // complete statement to install database
		$tables = $matches = $attributes = array();
		
		foreach ($db->splitSql($queries) as $query) // run all queries for non-existing tables
			if (JString::trim($query))
				$db->setQuery($query)->query();
		
		if (preg_match_all('/CREATE TABLE IF NOT EXISTS `([^`]*)` ([^;]*);/isU', $queries, $matches, PREG_SET_ORDER)) // search create table statement
			foreach ($matches as $match)
				if (preg_match_all('/(  `([^`]+)`[^,]+),/', $match[2], $attributes, PREG_SET_ORDER)) // search create table attribute statement
					foreach ($attributes as $attribute)
						$tables[$match[1]][$attribute[2]] = $attribute[1]; // fieldname => statement
		 
		foreach ($tables as $tablename => $attributes) { // update existing tables	
			$fields = $db->getTableColumns($tablename); // attribute list of existing table
			foreach ($attributes as $fieldname => $statement) { // process attribute list from new table statement
				if (! isset($fields[$fieldname])) // new attribute
					$db->setQuery("ALTER TABLE $tablename ADD $statement")->query();
				else
					unset($fields[$fieldname]); // attribute already exists
			}
			if (!empty($fields)) // attributes are not in the new table statement = drop them
				$db->setQuery("ALTER TABLE $tablename DROP " . implode(', DROP ', array_keys($fields)))->query();
		}
	}
}